@@ -7,7 +7,7 @@
  * This file is part of libSBML.  Please visit http://sbml.org for more
  * information about SBML, and the latest version of libSBML.
  *
- * Copyright (C) 2013-2015 jointly by the following organizations:
+ * Copyright (C) 2013-2016 jointly by the following organizations:
  *     1. California Institute of Technology, Pasadena, CA, USA
  *     2. EMBL European Bioinformatics Institute (EMBL-EBI), Hinxton, UK
  *     3. University of Heidelberg, Heidelberg, Germany
@@ -86,11 +86,7 @@ SBMLLevelVersionConverter::~SBMLLevelVersionConverter ()
 SBMLLevelVersionConverter& 
 SBMLLevelVersionConverter::operator=(const SBMLLevelVersionConverter& rhs)
 {  
-  if (&rhs == NULL)
-  {
-    throw SBMLConstructorException("Null argument to assignment operator");
-  }
-  else if(&rhs!=this)
+  if(&rhs!=this)
   {
     this->SBMLConverter::operator =(rhs);
   }
@@ -124,6 +120,8 @@ SBMLLevelVersionConverter::getDefaultProperties() const
                    "Whether validity should be strictly preserved");
     prop.addOption("setLevelAndVersion", true, 
                    "Convert the model to a given Level and Version of SBML");
+    prop.addOption("addDefaultUnits", true,
+                   "Whether default units should be added when converting to L3");
     delete sbmlns;
     init = true;
     return prop;
@@ -134,7 +132,7 @@ SBMLLevelVersionConverter::getDefaultProperties() const
 bool 
 SBMLLevelVersionConverter::matchesProperties(const ConversionProperties &props) const
 {
-  if (&props == NULL || !props.hasOption("setLevelAndVersion"))
+  if (!props.hasOption("setLevelAndVersion"))
     return false;
   return true;
 }
@@ -185,6 +183,23 @@ SBMLLevelVersionConverter::getValidityFlag()
   }
 }
 
+bool
+SBMLLevelVersionConverter::getAddDefaultUnits()
+{
+  if (getProperties() == NULL)
+  {
+    return true;
+  }
+  else if (getProperties()->hasOption("addDefaultUnits") == false)
+  {
+    return true;
+  }
+  else
+  {
+    return getProperties()->getBoolValue("addDefaultUnits");
+  }
+}
+
 
 int
 SBMLLevelVersionConverter::convert()
@@ -317,9 +332,9 @@ SBMLLevelVersionConverter::convert()
 
   if (currentModel != NULL)
   {
-    unsigned int origLevel;
-    unsigned int origVersion;
-    Model *origModel;
+    unsigned int origLevel = 0;
+    unsigned int origVersion = 0;
+    Model *origModel = NULL;
     if (strict)
     {
       /* here we are strict and only want to do
@@ -340,7 +355,7 @@ SBMLLevelVersionConverter::convert()
 
       if (strict)
       {
-        delete origModel;
+        if (origModel != NULL) delete origModel;
         mDocument->setApplicableValidators(origValidators);
         mDocument->updateSBMLNamespace("core", origLevel, origVersion);
       }
@@ -397,6 +412,8 @@ SBMLLevelVersionConverter::performConversion(bool strict, bool strictUnits,
   bool conversion = false;
  
   bool doConversion = false;
+
+  bool addDefaultUnits = getAddDefaultUnits();
   
   unsigned int currentLevel = mDocument->getLevel();
   unsigned int currentVersion = mDocument->getVersion();
@@ -489,7 +506,7 @@ SBMLLevelVersionConverter::performConversion(bool strict, bool strictUnits,
         currentModel->removeParameterRuleUnits(strict);
         currentModel->convertParametersToLocals(targetLevel, targetVersion);
         mDocument->updateSBMLNamespace("core", targetLevel, targetVersion);
-        currentModel->convertL1ToL3();
+        currentModel->convertL1ToL3(addDefaultUnits);
         conversion = true;
       }
       break;
@@ -753,7 +770,7 @@ SBMLLevelVersionConverter::performConversion(bool strict, bool strictUnits,
         }
         currentModel->convertParametersToLocals(targetLevel, targetVersion);
         mDocument->updateSBMLNamespace("core", targetLevel, targetVersion);
-        currentModel->convertL2ToL3(strict);
+        currentModel->convertL2ToL3(strict, addDefaultUnits);
         conversion = true;
       }
       break;
@@ -950,7 +967,6 @@ SBMLLevelVersionConverter::performConversion(bool strict, bool strictUnits,
   return conversion;
 
 }
-
 /** @endcond */
 
 
@@ -1082,7 +1098,6 @@ SBMLLevelVersionConverter::hasStrictSBO()
 
 
 /** @cond doxygenIgnored */
-
 unsigned int
 SBMLLevelVersionConverter::validateConvertedDocument()
 {
