@@ -7,7 +7,7 @@
  * This file is part of libSBML.  Please visit http://sbml.org for more
  * information about SBML, and the latest version of libSBML.
  *
- * Copyright (C) 2013-2015 jointly by the following organizations:
+ * Copyright (C) 2013-2016 jointly by the following organizations:
  *     1. California Institute of Technology, Pasadena, CA, USA
  *     2. EMBL European Bioinformatics Institute (EMBL-EBI), Hinxton, UK
  *     3. University of Heidelberg, Heidelberg, Germany
@@ -272,7 +272,6 @@ static void
 logError (XMLInputStream& stream, const XMLToken& element, SBMLErrorCode_t code,
           const std::string& msg = "")
 {
-  if (&element == NULL || &stream == NULL) return;
 
   SBMLNamespaces* ns = stream.getSBMLNamespaces();
   if (ns != NULL)
@@ -307,8 +306,6 @@ logError (XMLInputStream& stream, const XMLToken& element, SBMLErrorCode_t code,
 static const string
 trim (const string& s)
 {
-  if (&s == NULL) return s;
-
   static const string whitespace(" \t\r\n");
 
   string::size_type begin = s.find_first_not_of(whitespace);
@@ -334,7 +331,6 @@ trim (const string& s)
 static void
 checkFunctionArgs (ASTNode& node)
 {
-  if (&node == NULL) return;
 
   if (node.getNumChildren() == 1)
   {
@@ -366,7 +362,6 @@ checkFunctionArgs (ASTNode& node)
 static void
 reduceBinary (ASTNode& node)
 {
-  if (&node == NULL) return;
 
   if (node.getNumChildren() == 2)
   {
@@ -385,7 +380,6 @@ reduceBinary (ASTNode& node)
 static void
 setTypeCI (ASTNode& node, const XMLToken& element, XMLInputStream& stream)
 {
-  if (&node == NULL || &element == NULL || &stream == NULL) return;
 
   if (element.getName() == "csymbol")
   {
@@ -433,7 +427,6 @@ setTypeCI (ASTNode& node, const XMLToken& element, XMLInputStream& stream)
 static void
 setTypeCN (ASTNode& node, const XMLToken& element, XMLInputStream& stream)
 {
-  if (&node == NULL || &element == NULL || &stream == NULL) return;
 
   string type = "real";
   element.getAttributes().readInto("type", type);
@@ -562,7 +555,6 @@ setTypeCN (ASTNode& node, const XMLToken& element, XMLInputStream& stream)
 static void
 setTypeOther (ASTNode& node, const XMLToken& element, XMLInputStream& stream)
 {
-  if (&node == NULL || &element == NULL || &stream == NULL) return;
 
   static const int size = sizeof(MATHML_ELEMENTS) / sizeof(MATHML_ELEMENTS[0]);
   const char*      name = element.getName().c_str();
@@ -582,7 +574,6 @@ setTypeOther (ASTNode& node, const XMLToken& element, XMLInputStream& stream)
 static void
 setType (ASTNode& node, const XMLToken& element, XMLInputStream& stream)
 {
-  if (&node == NULL || &element == NULL || &stream == NULL) return;
 
   const string& name = element.getName();
 
@@ -618,7 +609,6 @@ setType (ASTNode& node, const XMLToken& element, XMLInputStream& stream)
 bool
 isMathMLNodeTag(const string& name)
 {
-  if (&name == NULL) return false;
 
   if ( name == "apply"
     || name == "cn"
@@ -647,7 +637,6 @@ static void
 readMathML (ASTNode& node, XMLInputStream& stream, std::string reqd_prefix,
             bool inRead)
 {
-  if (&node == NULL || &stream == NULL) return;
 
   std::string prefix;
   bool prefix_reqd = false;
@@ -793,6 +782,23 @@ readMathML (ASTNode& node, XMLInputStream& stream, std::string reqd_prefix,
        */
       if (elem.isEnd()) return;
 
+      // check for names that should not follow an apply
+      // but that will not create a node with a type that is
+      // always incorrect
+      stream.skipText();
+      std::string nextName = stream.peek().getName();
+      if (nextName == "bvar" || nextName == "piece" || nextName == "otherwise"
+        || nextName == "logbase" || nextName == "degree" 
+        || nextName == "lambda" || nextName == "semantics")
+      {
+        std::string message = "<";
+        message += nextName;
+        message += "> is not an operator and cannot be used directly following an";
+        message += " <apply> tag.";
+
+        logError(stream, elem, BadMathML, message);
+      }
+
       readMathML(node, stream, reqd_prefix, inRead);
 
       if (node.isName()) node.setType(AST_FUNCTION);
@@ -831,7 +837,18 @@ readMathML (ASTNode& node, XMLInputStream& stream, std::string reqd_prefix,
         return;
 
       }
-    }
+      else if (node.getType() == AST_FUNCTION_PIECEWISE) 
+      {
+        std::string message = "A <piecewise> element";
+        message += " is not an operator and cannot be used directly following an";
+        message += " <apply> tag.";
+
+        logError(stream, elem, BadMathML, message);
+
+        return;
+
+      }
+   }
     else if (name == "lambda")
     {
       node.setType(AST_LAMBDA);
@@ -885,14 +902,15 @@ readMathML (ASTNode& node, XMLInputStream& stream, std::string reqd_prefix,
        /* look to see whether a lambda is followed by an
        * appropriate tag
        */
+      std::string nextName = stream.peek().getName();
       if (elem.getName() == "lambda" 
-        && stream.peek().getName() != "lambda"
-        && stream.peek().getName() != "bvar")
+        && nextName != "lambda"
+        && nextName != "bvar")
       {
-        if ( !isMathMLNodeTag(stream.peek().getName()))
+        if ( !isMathMLNodeTag(nextName))
         {
           std::string message = "<";
-          message += stream.peek().getName();
+          message += nextName;
           message += "> cannot be used directly following a";
           message += " <bvar> element.";
 
@@ -906,7 +924,7 @@ readMathML (ASTNode& node, XMLInputStream& stream, std::string reqd_prefix,
        * dont want to add the child since this makes it look like
        * it has a bvar
        */
-      if (stream.peek().getName() == "math") 
+      if (nextName == "math") 
       {
         delete child;
         break;
@@ -916,7 +934,7 @@ readMathML (ASTNode& node, XMLInputStream& stream, std::string reqd_prefix,
         node.addChild(child, true);
       }
 
-      if (stream.peek().getName() == "piece" && stream.isGood()) 
+      if (nextName == "piece" && stream.isGood()) 
         stream.next();
     }
   }
@@ -1018,7 +1036,6 @@ writeAttributes(const ASTNode& node, XMLOutputStream& stream)
 static void
 writeCI (const ASTNode& node, XMLOutputStream& stream, SBMLNamespaces *sbmlns)
 {
-  if (&node == NULL || &stream == NULL) return;
 
   ASTNodeType_t type = node.getType();
 
@@ -1057,7 +1074,6 @@ writeCI (const ASTNode& node, XMLOutputStream& stream, SBMLNamespaces *sbmlns)
 static void
 writeCN (const ASTNode& node, XMLOutputStream& stream, SBMLNamespaces *sbmlns=NULL)
 {
-  if (&node == NULL || &stream == NULL) return;
 
   if ( node.isNaN() )
   {
@@ -1133,7 +1149,6 @@ writeCN (const ASTNode& node, XMLOutputStream& stream, SBMLNamespaces *sbmlns=NU
 static void
 writeConstant (const ASTNode& node, XMLOutputStream& stream)
 {
-  if (&node == NULL || &stream == NULL) return;
 
   switch ( node.getType() )
   {
@@ -1159,7 +1174,6 @@ writeConstant (const ASTNode& node, XMLOutputStream& stream)
 static void
 writeCSymbol (const ASTNode& node, XMLOutputStream& stream, SBMLNamespaces *sbmlns)
 {
-  if (&node == NULL || &stream == NULL) return;
 
   ASTNodeType_t type = node.getType();
   string url;
@@ -1191,7 +1205,6 @@ writeCSymbol (const ASTNode& node, XMLOutputStream& stream, SBMLNamespaces *sbml
 static void
 writeDouble (const double& value, XMLOutputStream& stream)
 {
-  if (&value == NULL || &stream == NULL) return;
 
   ostringstream output;
 
@@ -1229,7 +1242,6 @@ writeENotation (  const double&    mantissa
                 , long             exponent
                 , XMLOutputStream& stream )
 {
-  if (&mantissa == NULL || &stream == NULL) return;
 
   ostringstream output;
 
@@ -1264,7 +1276,6 @@ writeENotation (  const string&    mantissa
                 , const string&    exponent
                 , XMLOutputStream& stream )
 {
-  if (&mantissa == NULL || &exponent == NULL || &stream == NULL) return;
 
   static const string enotation = "e-notation";
   stream.writeAttribute("type", enotation);
@@ -1283,7 +1294,6 @@ writeENotation (  const string&    mantissa
 static void
 writeFunctionLog (const ASTNode& node, XMLOutputStream& stream, SBMLNamespaces *sbmlns)
 {
-  if (&node == NULL || &stream == NULL) return;
 
   if ( node.getNumChildren() > 1 )
   {
@@ -1306,7 +1316,6 @@ writeFunctionLog (const ASTNode& node, XMLOutputStream& stream, SBMLNamespaces *
 static void
 writeFunctionRoot (const ASTNode& node, XMLOutputStream& stream, SBMLNamespaces *sbmlns)
 {
-  if (&node == NULL || &stream == NULL) return;
 
   if ( node.getNumChildren() > 1 )
   {
@@ -1333,7 +1342,6 @@ writeFunctionRoot (const ASTNode& node, XMLOutputStream& stream, SBMLNamespaces
 static void
 writeFunction (const ASTNode& node, XMLOutputStream& stream, SBMLNamespaces *sbmlns)
 {
-  if (&node == NULL || &stream == NULL) return;
 
   ASTNodeType_t type        = node.getType();
   unsigned int  numChildren = node.getNumChildren();
@@ -1392,7 +1400,6 @@ writeFunction (const ASTNode& node, XMLOutputStream& stream, SBMLNamespaces *sbm
 static void
 writeLambda (const ASTNode& node, XMLOutputStream& stream, SBMLNamespaces *sbmlns)
 {
-  if (&node == NULL || &stream == NULL) return;
 
   bool bodyPresent = true;
   unsigned int bvars = node.getNumChildren() - 1;
@@ -1432,7 +1439,6 @@ writeLambda (const ASTNode& node, XMLOutputStream& stream, SBMLNamespaces *sbmln
 static void
 writeOperatorArgs (const ASTNode& node, XMLOutputStream& stream, SBMLNamespaces *sbmlns)
 {
-  if (&node == NULL || &stream == NULL) return;
 
   ASTNodeType_t type  = node.getType();
   ASTNode*      left  = node.getLeftChild();
@@ -1497,7 +1503,6 @@ writeOperatorArgs (const ASTNode& node, XMLOutputStream& stream, SBMLNamespaces
 static void
 writeOperator (const ASTNode& node, XMLOutputStream& stream, SBMLNamespaces *sbmlns)
 {
-  if (&node == NULL || &stream == NULL) return;
 
   stream.startElement("apply");
 
@@ -1530,7 +1535,6 @@ writeOperator (const ASTNode& node, XMLOutputStream& stream, SBMLNamespaces *sbm
 static void
 writePiecewise (const ASTNode& node, XMLOutputStream& stream, SBMLNamespaces *sbmlns)
 {
-  if (&node == NULL || &stream == NULL) return;
 
   unsigned int numChildren = node.getNumChildren();
   unsigned int numPieces   = numChildren;
@@ -1573,7 +1577,6 @@ writePiecewise (const ASTNode& node, XMLOutputStream& stream, SBMLNamespaces *sb
 static void
 writeSemantics(const ASTNode& node, XMLOutputStream& stream, bool &inSemantics, SBMLNamespaces *sbmlns)
 {
-  if (&node == NULL || &stream == NULL || &inSemantics == NULL) return;
 
   inSemantics = true;
   stream.startElement("semantics");
@@ -1600,7 +1603,6 @@ writeSemantics(const ASTNode& node, XMLOutputStream& stream, bool &inSemantics,
 static void
 writeNode (const ASTNode& node, XMLOutputStream& stream, SBMLNamespaces *sbmlns)
 {
-  if (&node == NULL || &stream == NULL) return;
 
   static bool inSemantics = false;
   
@@ -1626,7 +1628,6 @@ LIBSBML_EXTERN
 ASTNode*
 readMathML (XMLInputStream& stream, std::string reqd_prefix, bool inRead)
 {
-  if (&stream == NULL) return NULL;
 
   std::string prefix;
   bool prefix_reqd = false;
@@ -1720,7 +1721,6 @@ LIBSBML_EXTERN
 void
 writeMathML (const ASTNode* node, XMLOutputStream& stream, SBMLNamespaces *sbmlns)
 {
-  if (node == NULL || &stream == NULL) return;
 
   static const string uri = "http://www.w3.org/1998/Math/MathML";
 
