@@ -8,7 +8,7 @@
  * This file is part of libSBML.  Please visit http://sbml.org for more
  * information about SBML, and the latest version of libSBML.
  *
- * Copyright (C) 2013-2015 jointly by the following organizations:
+ * Copyright (C) 2013-2016 jointly by the following organizations:
  *     1. California Institute of Technology, Pasadena, CA, USA
  *     2. EMBL European Bioinformatics Institute (EMBL-EBI), Hinxton, UK
  *     3. University of Heidelberg, Heidelberg, Germany
@@ -40,6 +40,11 @@
 #include <sbml/util/ElementFilter.h>
 #include <sbml/util/PrefixTransformer.h>
 
+#ifdef LIBSBML_HAS_PACKAGE_FBC
+#include <sbml/packages/fbc/extension/FbcModelPlugin.h>
+#endif // LIBSBML_HAS_PACKAGE_FBC
+
+
 using namespace std;
 
 LIBSBML_CPP_NAMESPACE_BEGIN
@@ -544,6 +549,23 @@ Model* CompModelPlugin::flattenModel() const
       delete flat;
       return NULL;
     }
+#ifdef LIBSBML_HAS_PACKAGE_FBC
+    // for an fbc v2 model we need to check that the model element 
+    // in the flat document has the fbc:strict attribute
+    // note this can happen if the parent document did not have fbc but
+    // included a submodel from an external document that did
+    if (SBMLExtensionRegistry::isPackageEnabled("fbc"))
+    {
+      FbcModelPlugin *mplugin = static_cast<FbcModelPlugin*>
+        (flat->getPlugin("fbc"));
+      if (mplugin != NULL && mplugin->getPackageVersion() == 2
+        && mplugin->isSetStrict() == false)
+      {
+        mplugin->setStrict(false);
+      }
+    }
+#endif // LIBSBML_HAS_PACKAGE_FBC
+
   }
 
   // Now we clear the saved referenced elements in the local Port objects, 
@@ -1317,8 +1339,7 @@ int CompModelPlugin::removeCollectedElements(set<SBase*>* removed, set<SBase*>*
 }
 
   
-  /** @cond doxygenLibsbmlInternal */
-
+/** @cond doxygenLibsbmlInternal */
 bool 
 CompModelPlugin::accept(SBMLVisitor& v) const
 {
@@ -1338,7 +1359,6 @@ CompModelPlugin::accept(SBMLVisitor& v) const
 
   return true;
 }
-
 /** @endcond */
 
 void 
