@@ -9,7 +9,7 @@
  * This file is part of libSBML.  Please visit http://sbml.org for more
  * information about SBML, and the latest version of libSBML.
  *
- * Copyright (C) 2013-2014 jointly by the following organizations:
+ * Copyright (C) 2013-2016 jointly by the following organizations:
  *     1. California Institute of Technology, Pasadena, CA, USA
  *     2. EMBL European Bioinformatics Institute (EMBL-EBI), Hinxton, UK
  *     3. University of Heidelberg, Heidelberg, Germany
@@ -96,9 +96,7 @@
 #include <sbml/validator/ConstraintMacros.h>
 
 /** @cond doxygenIgnored */
-
 using namespace std;
-
 /** @endcond */
 
 START_CONSTRAINT (20217, Model, x)
@@ -107,6 +105,9 @@ START_CONSTRAINT (20217, Model, x)
   pre( m.getLevel() > 2);
   pre( m.isSetTimeUnits());
 
+  msg = "The 'timeUnits' attribute of the <model> is '" + m.getTimeUnits()
+    + "', which does not comply.";
+
   const string&         units = m.getTimeUnits();
   const UnitDefinition* defn  = m.getUnitDefinition(units);
 
@@ -124,6 +125,9 @@ START_CONSTRAINT (20218, Model, x)
   pre( m.getLevel() > 2);
   pre( m.isSetVolumeUnits());
 
+  msg = "The 'volumeUnits' attribute of the <model> is '" + m.getVolumeUnits()
+    + "', which does not comply.";
+
   const string&         units = m.getVolumeUnits();
   const UnitDefinition* defn  = m.getUnitDefinition(units);
 
@@ -141,6 +145,9 @@ START_CONSTRAINT (20219, Model, x)
   pre( m.getLevel() > 2);
   pre( m.isSetAreaUnits());
 
+  msg = "The 'areaUnits' attribute of the <model> is '" + m.getAreaUnits()
+    + "', which does not comply.";
+
   const string&         units = m.getAreaUnits();
   const UnitDefinition* defn  = m.getUnitDefinition(units);
 
@@ -157,6 +164,9 @@ START_CONSTRAINT (20220, Model, x)
   pre( m.getLevel() > 2);
   pre( m.isSetLengthUnits());
 
+  msg = "The 'lengthUnits' attribute of the <model> is '" + m.getLengthUnits()
+    + "', which does not comply.";
+
   const string&         units = m.getLengthUnits();
   const UnitDefinition* defn  = m.getUnitDefinition(units);
 
@@ -174,6 +184,9 @@ START_CONSTRAINT (20221, Model, x)
   pre( m.getLevel() > 2);
   pre( m.isSetExtentUnits());
 
+  msg = "The 'extentUnits' attribute of the <model> is '" + m.getExtentUnits()
+    + "', which does not comply.";
+
   const string&         units = m.getExtentUnits();
   const UnitDefinition* defn  = m.getUnitDefinition(units);
 
@@ -217,17 +230,32 @@ START_CONSTRAINT (20509, Compartment, c)
   const UnitDefinition* defn  = m.getUnitDefinition(units);
 
   /* dimensionless is allowable in L2V2 */
-  if ( c.getLevel() == 1
-    || (c.getLevel() == 2 && c.getVersion() == 1))
+  if ( c.getLevel() == 1)
   {
     inv_or( units == "volume" );
     inv_or( units == "litre"  );
-    inv_or( units == "liter" && c.getLevel() == 1 );
+    inv_or( units == "liter"  );
     inv_or( defn  != NULL && defn->isVariantOfVolume() );
   }
+  else if (c.getLevel() == 2)
+  {
+    if (c.getVersion() == 1)
+    {
+    inv_or( units == "volume" );
+    inv_or( units == "litre"  );
+    inv_or( defn  != NULL && defn->isVariantOfVolume() );
+    }
+    else
+    {
+      inv_or( units == "volume" );
+      inv_or( units == "litre"  );
+      inv_or( units == "dimensionless"  );
+      inv_or( defn  != NULL && defn->isVariantOfVolume() );
+      inv_or( defn  != NULL && defn->isVariantOfDimensionless() );
+    }
+  }
   else
   {
-    inv_or( units == "volume" );
     inv_or( units == "litre"  );
     inv_or( units == "dimensionless"  );
     inv_or( defn  != NULL && defn->isVariantOfVolume() );
@@ -248,7 +276,8 @@ START_CONSTRAINT (20608, Species, s)
       "of the following: 'substance', "
       "or the identifier of a <unitDefinition> derived from "
       "'mole' (with an 'exponent' of '1') or 'item' (with an 'exponent' "
-      "of '1').";
+      "of '1').  The current value ('" + s.getSubstanceUnits() + "') is "
+      "not allowed.";
   }
   else if (s.getLevel() == 2)
   {
@@ -259,7 +288,8 @@ START_CONSTRAINT (20608, Species, s)
         "of the following: 'substance', 'mole' or 'item' "
         "or the identifier of a <unitDefinition> derived from "
         "'mole' (with an 'exponent' of '1') or 'item' (with an 'exponent' "
-        "of '1').";
+        "of '1').  The current value ('" + s.getSubstanceUnits() + "') is "
+      "not allowed.";
     }
     else
     {
@@ -269,18 +299,21 @@ START_CONSTRAINT (20608, Species, s)
         "'dimensionless', or the identifier of a <unitDefinition> derived from "
         "'mole' (with an 'exponent' of '1'), 'item' (with an 'exponent' of '1')"
         ", 'gram' (with an 'exponent' of '1'), 'kilogram' (with an 'exponent' "
-        "of '1'), or 'dimensionless'.";
+        "of '1'), or 'dimensionless'.  The current value ('" + s.getSubstanceUnits() 
+        + "') is not allowed.";
     }
   }
   else
   {
     msg =
       "The value of a <species>'s 'substanceUnits' attribute can only be one "
-      "of the following: 'substance', 'mole', 'item', 'gram', 'kilogram', "
+      "of the following: 'mole', 'item', 'gram', 'kilogram', "
       "'dimensionless', 'avogadro' or the identifier of a <unitDefinition> derived from "
       "'mole' (with an 'exponent' of '1'), 'item' (with an 'exponent' of '1')"
       ", 'gram' (with an 'exponent' of '1'), 'kilogram' (with an 'exponent' "
-      "of '1'), 'avogadro' (with an 'exponent' of '1') or 'dimensionless'.";
+      "of '1'), 'avogadro' (with an 'exponent' of '1') or 'dimensionless'.  The "
+      "current value ('" + s.getSubstanceUnits() + "') is "
+      "not allowed.";
   }
 
   const string&         units = s.getSubstanceUnits();
@@ -309,7 +342,6 @@ START_CONSTRAINT (20608, Species, s)
   }
   else if (s.getLevel() == 3)
   {
-    inv_or( units == "substance"      );
     inv_or( units == "item"           );
     inv_or( units == "mole"           );
     inv_or( units == "dimensionless"  );
@@ -328,6 +360,8 @@ START_CONSTRAINT (20701, Parameter, p)
 {
   pre( p.isSetUnits() );
 
+  msg = "The 'units' attribute of the <parameter> is '" + p.getUnits() 
+    + "', which does not comply.";
   //msg =
   //  "The 'units' in a <parameter> definition must be a value chosen from "
   //  "among the following: a value from the 'UnitKind' enumeration (e.g., "
@@ -350,6 +384,8 @@ START_CONSTRAINT (99130, Model, x)
   pre( m.getLevel() > 2);
   pre( m.isSetSubstanceUnits());
 
+  msg = "The 'substanceUnits' attribute of the <model> is '" + m.getSubstanceUnits() 
+    + "', which does not comply.";
   const string&         units = m.getSubstanceUnits();
   const UnitDefinition* defn  = m.getUnitDefinition(units);
 
@@ -569,8 +605,6 @@ START_CONSTRAINT (99508, Parameter, p)
   bool isNotNull = ud != NULL;
   int numUnits = isNotNull ? ud->getNumUnits() : 0;
   
-  delete(ud);
-  
   if (!isNotNull)  
 
   pre ( isNotNull);
@@ -707,7 +741,8 @@ START_CONSTRAINT (10511, AssignmentRule, ar)
       "the units of the rule's right-hand side must be consistent with the "
       "units of that <compartment>'s volume. Expected units are ";
     msg += UnitDefinition::printUnits(variableUnits->getUnitDefinition());
-    msg += " but the units returned by the <compartmentVolumeRule>'s formula are ";
+    msg += " but the units returned by the <compartmentVolumeRule> with variable '";
+    msg += variable + "'s formula are ";
     msg += UnitDefinition::printUnits(formulaUnits->getUnitDefinition());
     msg += ".";
   }
@@ -715,7 +750,8 @@ START_CONSTRAINT (10511, AssignmentRule, ar)
   {
     msg =  " Expected units are ";
     msg += UnitDefinition::printUnits(variableUnits->getUnitDefinition());
-    msg += " but the units returned by the <assignmentRule>'s <math> expression are ";
+    msg += " but the units returned by the <math> expression of the ";
+    msg += "<assignmentRule> with variable '" + variable + "' are ";
     msg += UnitDefinition::printUnits(formulaUnits->getUnitDefinition());
     msg += ".";
   }
@@ -776,7 +812,8 @@ START_CONSTRAINT (10512, AssignmentRule, ar)
       "the units of the rule's right-hand side must be consistent with the "
       "units of that <species> quantity. Expected units are ";
     msg += UnitDefinition::printUnits(variableUnits->getUnitDefinition());
-    msg += " but the units returned by the <speciesConcentrationRule>'s formula are ";
+    msg += " but the units returned by the <speciesConcentrationRule> with variable '";
+    msg += variable + "'s formula are ";
     msg += UnitDefinition::printUnits(formulaUnits->getUnitDefinition());
     msg += ".";
   }
@@ -784,7 +821,8 @@ START_CONSTRAINT (10512, AssignmentRule, ar)
   {
     msg =  " Expected units are ";
     msg += UnitDefinition::printUnits(variableUnits->getUnitDefinition());
-    msg += " but the units returned by the <assignmentRule>'s <math> expression are ";
+    msg += " but the units returned by the <math> expression of the ";
+    msg += "<assignmentRule> with variable '" + variable + "' are ";
     msg += UnitDefinition::printUnits(formulaUnits->getUnitDefinition());
     msg += ".";
   }
@@ -855,7 +893,8 @@ START_CONSTRAINT (10513, AssignmentRule, ar)
       "the units of the rule's right-hand side must be consistent with the "
       "units declared for that <parameter>. Expected units are ";
     msg += UnitDefinition::printUnits(variableUnits->getUnitDefinition());
-    msg += " but the units returned by the <parameterRule>'s formula are ";
+    msg += " but the units returned by the <parameterRule> with variable '";
+    msg += variable + "'s formula are ";
     msg += UnitDefinition::printUnits(formulaUnits->getUnitDefinition());
     msg += ".";
   }
@@ -863,7 +902,8 @@ START_CONSTRAINT (10513, AssignmentRule, ar)
   {
     msg =  " Expected units are ";
     msg += UnitDefinition::printUnits(variableUnits->getUnitDefinition());
-    msg += " but the units returned by the <assignmentRule>'s <math> expression are ";
+    msg += " but the units returned by the <math> expression of the ";
+    msg += "<assignmentRule> with variable '" + variable + "' are ";
     msg += UnitDefinition::printUnits(formulaUnits->getUnitDefinition());
     msg += ".";
   }
@@ -901,7 +941,8 @@ START_CONSTRAINT (10514, AssignmentRule, ar)
         formulaUnits->getCanIgnoreUndeclaredUnits()));
 
   msg =  " Expected units are dimensionless";
-  msg += " but the units returned by the <assignmentRule>'s <math> expression are ";
+  msg += " but the units returned by the <math> expression of the ";
+  msg += "<assignmentRule> with variable '" + variable + "' are ";
   msg += UnitDefinition::printUnits(formulaUnits->getUnitDefinition());
   msg += ".";
   
@@ -944,7 +985,8 @@ START_CONSTRAINT (10521, InitialAssignment, ia)
 
   msg =  "Expected units are ";
   msg += UnitDefinition::printUnits(variableUnits->getUnitDefinition());
-  msg += " but the units returned by the <initialAssignment>'s <math> expression are ";
+  msg += " but the units returned by the <math> expression of the ";
+  msg += "<initialAssignment> with symbol '" + variable + "' are ";
   msg += UnitDefinition::printUnits(formulaUnits->getUnitDefinition());
   msg += ".";
 
@@ -988,7 +1030,8 @@ START_CONSTRAINT (10522, InitialAssignment, ia)
 
   msg =  "Expected units are ";
   msg += UnitDefinition::printUnits(variableUnits->getUnitDefinition());
-  msg += " but the units returned by the <initialAssignment>'s <math> expression are ";
+  msg += " but the units returned by the <math> expression of the ";
+  msg += "<initialAssignment> with symbol '" + variable + "' are ";
   msg += UnitDefinition::printUnits(formulaUnits->getUnitDefinition());
   msg += ".";
 
@@ -1030,7 +1073,8 @@ START_CONSTRAINT (10523, InitialAssignment, ia)
 
   msg =  "Expected units are ";
   msg += UnitDefinition::printUnits(variableUnits->getUnitDefinition());
-  msg += " but the units returned by the <initialAssignment>'s <math> expression are ";
+  msg += " but the units returned by the <math> expression of the ";
+  msg += "<initialAssignment> with symbol '" + variable + "' are ";
   msg += UnitDefinition::printUnits(formulaUnits->getUnitDefinition());
   msg += ".";
 
@@ -1067,7 +1111,8 @@ START_CONSTRAINT (10524, InitialAssignment, ia)
         formulaUnits->getCanIgnoreUndeclaredUnits()));
 
   msg =  " Expected units are dimensionless";
-  msg += " but the units returned by the <initialAssignment>'s <math> expression are ";
+  msg += " but the units returned by the <math> expression of the ";
+  msg += "<initialAssignment> with symbol '" + variable + "' are ";
   msg += UnitDefinition::printUnits(formulaUnits->getUnitDefinition());
   msg += ".";
   
@@ -1139,7 +1184,8 @@ START_CONSTRAINT (10531, RateRule, rr)
     "for the compartment volume) the default units for that compartment, and "
     "_time_ refers to the units of time for the model. Expected units are ";    
     msg += UnitDefinition::printUnits(variableUnits->getPerTimeUnitDefinition());
-    msg += " but the units returned by the <compartmentVolumeRule>'s formula are ";
+    msg += " but the units returned by the <compartmentVolumeRule> with variable '";
+    msg += variable + "'s formula are ";
     msg += UnitDefinition::printUnits(formulaUnits->getUnitDefinition());
     msg += ".";
   }
@@ -1147,7 +1193,8 @@ START_CONSTRAINT (10531, RateRule, rr)
   {
     msg =  " Expected units are ";
     msg += UnitDefinition::printUnits(variableUnits->getPerTimeUnitDefinition());
-    msg += " but the units returned by the <rateRule>'s <math> expression are ";
+    msg += " but the units returned by the <math> expression of the ";
+    msg += "<rateRule> with variable '" + variable + "' are ";
     msg += UnitDefinition::printUnits(formulaUnits->getUnitDefinition());
     msg += ".";
   }
@@ -1215,7 +1262,8 @@ START_CONSTRAINT (10532, RateRule, rr)
     "time_, where _x_ is the units of that species' quantity, and _time_ "
     "refers to the units of time for the model. Expected units are ";    
     msg += UnitDefinition::printUnits(variableUnits->getPerTimeUnitDefinition());
-    msg += " but the units returned by the <speciesConcentrationRule>'s formula are ";
+    msg += " but the units returned by the <speciesConcentrationRule> with variable '";
+    msg += variable + "'s formula are ";
     msg += UnitDefinition::printUnits(formulaUnits->getUnitDefinition());
     msg += ".";
   }
@@ -1223,7 +1271,8 @@ START_CONSTRAINT (10532, RateRule, rr)
   {
     msg =  " Expected units are ";
     msg += UnitDefinition::printUnits(variableUnits->getPerTimeUnitDefinition());
-    msg += " but the units returned by the <rateRule>'s <math> expression are ";
+    msg += " but the units returned by the <math> expression of the ";
+    msg += "<rateRule> with variable '" + variable + "' are ";
     msg += UnitDefinition::printUnits(formulaUnits->getUnitDefinition());
     msg += ".";
   }
@@ -1302,7 +1351,8 @@ START_CONSTRAINT (10533, RateRule, rr)
     "time_, where _x_ is the 'units' in that <parameter> definition, and "
     "_time_ refers to the units of time for the model. Expected units are ";    
     msg += UnitDefinition::printUnits(variableUnits->getPerTimeUnitDefinition());
-    msg += " but the units returned by the <parameterRule>'s formula are ";
+    msg += " but the units returned by the <parameterRule> with variable '";
+    msg += variable + "'s formula are ";
     msg += UnitDefinition::printUnits(formulaUnits->getUnitDefinition());
     msg += ".";
   }
@@ -1310,7 +1360,8 @@ START_CONSTRAINT (10533, RateRule, rr)
   {
     msg =  " Expected units are ";
     msg += UnitDefinition::printUnits(variableUnits->getPerTimeUnitDefinition());
-    msg += " but the units returned by the <rateRule>'s <math> expression are ";
+    msg += " but the units returned by the <math> expression of the ";
+    msg += "<rateRule> with variable '" + variable + "' are ";
     msg += UnitDefinition::printUnits(formulaUnits->getUnitDefinition());
     msg += ".";
   }
@@ -1353,7 +1404,8 @@ START_CONSTRAINT (10534, RateRule, rr)
         formulaUnits->getCanIgnoreUndeclaredUnits()));
 
   msg =  " Expected units are dimensionless per time";
-  msg += " but the units returned by the <rateRule>'s <math> expression are ";
+  msg += " but the units returned by the <math> expression of the ";
+  msg += "<rateRule> with variable '" + variable + "' are ";
   msg += UnitDefinition::printUnits(formulaUnits->getUnitDefinition());
   msg += ".";
   
@@ -1393,6 +1445,7 @@ START_CONSTRAINT (10541, KineticLaw, kl)
     || (variableUnits->getContainsUndeclaredUnits() &&
         variableUnits->getCanIgnoreUndeclaredUnits()));
 
+  const SBase* rxn = kl.getAncestorOfType(SBML_REACTION);
   if (m.getLevel() > 2)
   {
   msg =
@@ -1404,7 +1457,11 @@ START_CONSTRAINT (10541, KineticLaw, kl)
   msg =  "Expected units are ";
   }
   msg += UnitDefinition::printUnits(variableUnits->getUnitDefinition());
-  msg += " but the units returned by the <kineticLaw>'s <math> expression are ";
+  msg += " but the units returned by the <math> expression in the <kineticLaw> ";
+  if (rxn && rxn->isSetId()) {
+    msg += "(from the <reaction> with id '" + rxn->getId() + "') ";
+  }
+  msg += "are ";
   msg += UnitDefinition::printUnits(formulaUnits->getUnitDefinition());
   msg += ".";
 
@@ -1484,7 +1541,11 @@ START_CONSTRAINT (10551, Event, e)
 
   msg =  "Expected units are ";
   msg += UnitDefinition::printUnits(formulaUnits->getEventTimeUnitDefinition());
-  msg += " but the units returned by the <event>'s <delay> are ";
+  msg += " but the units returned by the <delay> from the <event> ";
+  if (e.isSetId()) {
+    msg += "with id '" + e.getId() + "' ";
+  }
+  msg += "are ";
   msg += UnitDefinition::printUnits(formulaUnits->getUnitDefinition());
   msg += ".";
 
@@ -1529,9 +1590,15 @@ START_CONSTRAINT (10561, EventAssignment, ea)
     || (formulaUnits->getContainsUndeclaredUnits() &&
         formulaUnits->getCanIgnoreUndeclaredUnits()));
 
+  const SBase* e = ea.getAncestorOfType(SBML_EVENT);
   msg =  "Expected units are ";
   msg += UnitDefinition::printUnits(variableUnits->getUnitDefinition());
-  msg += " but the units returned by the <eventAssignment>'s <math> expression are ";
+  msg += " but the units returned by the <math> expression of the ";
+  msg += "<eventAssignment> with variable '" + variable + "' ";
+  if (e && e->isSetId()) {
+    msg += " from the <event> with id '" + e->getId() + "' ";
+  }
+  msg += "are ";
   msg += UnitDefinition::printUnits(formulaUnits->getUnitDefinition());
   msg += ".";
 
@@ -1578,7 +1645,8 @@ START_CONSTRAINT (10562, EventAssignment, ea)
 
   msg =  "Expected units are ";
   msg += UnitDefinition::printUnits(variableUnits->getUnitDefinition());
-  msg += " but the units returned by the <eventAssignment>'s <math> expression are ";
+  msg += " but the units returned by the <math> expression from the ";
+  msg += "<eventAssignment> with variable '" + variable + "' are ";
   msg += UnitDefinition::printUnits(formulaUnits->getUnitDefinition());
   msg += ".";
 
@@ -1623,7 +1691,8 @@ START_CONSTRAINT (10563, EventAssignment, ea)
 
   msg =  "Expected units are ";
   msg += UnitDefinition::printUnits(variableUnits->getUnitDefinition());
-  msg += " but the units returned by the <eventAssignment>'s <math> expression are ";
+  msg += " but the units returned by the <math> expression from the ";
+  msg += "<eventAssignment> with variable '" + variable + "' are ";
   msg += UnitDefinition::printUnits(formulaUnits->getUnitDefinition());
   msg += ".";
 
@@ -1662,7 +1731,8 @@ START_CONSTRAINT (10564, EventAssignment, ea)
         formulaUnits->getCanIgnoreUndeclaredUnits()));
 
   msg =  " Expected units are dimensionless";
-  msg += " but the units returned by the <eventAssignment>'s <math> expression are ";
+  msg += " but the units returned by the <math> expression from the ";
+  msg += "<eventAssignment> with variable '" + variable + "' are ";
   msg += UnitDefinition::printUnits(formulaUnits->getUnitDefinition());
   msg += ".";
   
@@ -1731,6 +1801,10 @@ END_CONSTRAINT
 START_CONSTRAINT (20518, Compartment, c)
 {
   pre( c.getLevel() > 2);
+  if (c.isSetId())
+  {
+    msg = "The <compartment> '" + c.getId() + "' has no discernable units.";
+  }
 
   inv( !(!(c.isSetUnits()) && !(c.isSetSpatialDimensions())));
 }
@@ -1742,6 +1816,11 @@ START_CONSTRAINT (20616, Species, s)
   pre( s.getLevel() > 2);
   pre( !(s.isSetSubstanceUnits()));
 
+  msg = "The <species> ";
+  if (s.isSetId()) {
+    msg += "with id '" + s.getId() + "' ";
+  }
+  msg += "does not have a substanceUnits attribute, nor does its enclosing <model>.";
   inv( m.isSetSubstanceUnits());
 }
 END_CONSTRAINT
@@ -1751,10 +1830,13 @@ START_CONSTRAINT (20702, Parameter, p)
 {
   pre( p.getLevel() > 2 );
 
+  msg = "The <parameter> ";
+  if (p.isSetId()) {
+    msg += "with id '" + p.getId() + "' ";
+  }
+  msg += "does not have a 'units' attribute.";
   inv( p.isSetUnits() );
 }
 END_CONSTRAINT
-
-
 /** @endcond */
 
